@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -60,7 +60,8 @@ yf::CQLtoRPN::~CQLtoRPN()
 {  // must have a destructor because of boost::scoped_ptr
 }
 
-void yf::CQLtoRPN::configure(const xmlNode *xmlnode, bool test_only)
+void yf::CQLtoRPN::configure(const xmlNode *xmlnode, bool test_only,
+                             const char *path)
 {
     m_p->configure(xmlnode);
 }
