@@ -54,6 +54,7 @@ static void sig_usr1_handler(int s)
 {
     yaz_log(YLOG_LOG, "metaproxy received SIGUSR1");
     routerp->stop();
+    yaz_daemon_stop();
 }
 
 static void sig_term_handler(int s)
@@ -69,7 +70,7 @@ static void work_common(void *data)
 {
 #if HAVE_UNISTD_H
     process_group = getpgid(0); // save process group ID
-    
+
     signal(SIGTERM, sig_term_handler);
     signal(SIGUSR1, sig_usr1_handler);
 #endif
@@ -78,7 +79,7 @@ static void work_common(void *data)
 
     mp::Package pack;
     pack.router(*routerp).move();
-    yaz_log(YLOG_LOG, "metaproxy stop");
+    yaz_log(YLOG_LOG, "metaproxy stop"); /* only for graceful stop */
     _exit(0);
 }
 
@@ -184,12 +185,13 @@ static int sc_main(
         return 1;
     }
     
-    yaz_log(YLOG_LOG, "Metaproxy start " VERSION
+    yaz_log(YLOG_LOG, "metaproxy start " VERSION
 #ifdef VERSION_SHA1
             " " VERSION_SHA1
 #endif
         );
     
+    yaz_log_xml_errors(0, YLOG_LOG);
     xmlDocPtr doc = xmlReadFile(fname,
                                 NULL, 
                                 XML_PARSE_XINCLUDE + XML_PARSE_NOBLANKS
@@ -197,7 +199,7 @@ static int sc_main(
     
     if (!doc)
     {
-        yaz_log (YLOG_FATAL,"XML parsing failed");
+        yaz_log(YLOG_FATAL,"XML parsing failed");
         return 1;
     }
     // and perform Xinclude then
@@ -234,7 +236,7 @@ static int sc_main(
         }
     }
     catch (std::logic_error &e) {
-        yaz_log (YLOG_FATAL,"std::logic error: %s" , e.what() );
+        yaz_log(YLOG_FATAL,"std::logic error: %s" , e.what() );
         ret = 1;
     }
     catch (std::runtime_error &e) {
@@ -265,8 +267,6 @@ int main(int argc, char **argv)
     exit(ret);
 }
 
-
-
 /*
  * Local variables:
  * c-basic-offset: 4
