@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -34,6 +34,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include <stdio.h>
 
 #include <deque>
+#include <sstream>
 
 #include <yazpp/socket-observer.h>
 #include <yaz/log.h>
@@ -68,6 +69,7 @@ namespace metaproxy_1 {
         std::deque<IThreadPoolMsg *> m_output;
         bool m_stop_flag;
         unsigned m_no_threads;
+        unsigned m_no_threads_waiting;
     };
     const unsigned int queue_size_per_thread = 64;
 }
@@ -100,6 +102,7 @@ ThreadPoolSocketObserver::ThreadPoolSocketObserver(
 
     m_p->m_stop_flag = false;
     m_p->m_no_threads = no_threads;
+    m_p->m_no_threads_waiting = 0;
     int i;
     for (i = 0; i<no_threads; i++)
     {
@@ -145,8 +148,21 @@ void ThreadPoolSocketObserver::socketNotify(int event)
             out = m_p->m_output.front();
             m_p->m_output.pop_front();
         }
+
+
         if (out)
-            out->result();
+        {
+            std::ostringstream os;
+            {
+                boost::mutex::scoped_lock input_lock(m_p->m_mutex_input_data);
+                os  << "tbusy/total " <<
+                    m_p->m_no_threads - m_p->m_no_threads_waiting <<
+                    "/" << m_p->m_no_threads
+                    << " queue in/out " << m_p->m_input.size() << "/"
+                    << m_p->m_output.size();
+            }
+            out->result(os.str().c_str());
+        }
     }
 }
 
@@ -157,8 +173,10 @@ void ThreadPoolSocketObserver::run(void *p)
         IThreadPoolMsg *in = 0;
         {
             boost::mutex::scoped_lock input_lock(m_p->m_mutex_input_data);
+            m_p->m_no_threads_waiting++;
             while (!m_p->m_stop_flag && m_p->m_input.size() == 0)
                 m_p->m_cond_input_data.wait(input_lock);
+            m_p->m_no_threads_waiting--;
             if (m_p->m_stop_flag)
                 break;
             
@@ -188,14 +206,30 @@ void ThreadPoolSocketObserver::run(void *p)
     }
 }
 
+void ThreadPoolSocketObserver::cleanup(IThreadPoolMsg *m, void *info)
+{
+    boost::mutex::scoped_lock input_lock(m_p->m_mutex_input_data);
+
+    std::deque<IThreadPoolMsg *>::iterator it = m_p->m_input.begin();
+    while (it != m_p->m_input.end())
+    {
+        if ((*it)->cleanup(info))
+            it = m_p->m_input.erase(it);
+        else
+            it++;
+    }
+}
+
 void ThreadPoolSocketObserver::put(IThreadPoolMsg *m)
 {
     boost::mutex::scoped_lock input_lock(m_p->m_mutex_input_data);
+
     while (m_p->m_input.size() >= m_p->m_no_threads * queue_size_per_thread)
         m_p->m_cond_input_full.wait(input_lock);
     m_p->m_input.push_back(m);
     m_p->m_cond_input_data.notify_one();
 }
+
 /*
  * Local variables:
  * c-basic-offset: 4
