@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -16,11 +16,12 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#include "config.hpp"
+
 #include <metaproxy/xmlutil.hpp>
 
 #include <string.h>
 
-
 namespace mp = metaproxy_1;
 // Doxygen doesn't like mp::xml, so we use this instead
 namespace mp_xml = metaproxy_1::xml;
@@ -45,9 +46,12 @@ std::string mp_xml::get_text(const xmlNode *ptr)
 
 bool mp_xml::get_bool(const xmlNode *ptr, bool default_value)
 {
+    if (ptr && ptr->type != XML_TEXT_NODE)
+        ptr = ptr->children;
     if (ptr && ptr->type == XML_TEXT_NODE && ptr->content)
     {
-        if (!strcmp((const char *) ptr->content, "true"))
+        if (!strcmp((const char *) ptr->content, "true")
+            || !strcmp((const char *) ptr->content, "1"))
             return true;
         else
             return false;
@@ -56,7 +60,9 @@ bool mp_xml::get_bool(const xmlNode *ptr, bool default_value)
 }
 
 int mp_xml::get_int(const xmlNode *ptr, int default_value)
-{
+{ 
+    if (ptr && ptr->type != XML_TEXT_NODE)
+        ptr = ptr->children;
     if (ptr && ptr->type == XML_TEXT_NODE && ptr->content)
     {
         return atoi((const char *) ptr->content);
@@ -143,7 +149,7 @@ bool mp_xml::check_element_mp(const xmlNode *ptr,
     return true;
 }
 
-std::string mp_xml::get_route(const xmlNode *node)
+std::string mp_xml::get_route(const xmlNode *node, std::string &auth)
 {
     std::string route_value;
     if (node)
@@ -159,6 +165,8 @@ std::string mp_xml::get_route(const xmlNode *node)
             
             if (name == "route")
                 route_value = value;
+            else if (name == "auth")
+                auth = value;
             else
                 throw XMLError("Only attribute route allowed"
                                " in " + std::string((const char *)node->name)
@@ -168,6 +176,11 @@ std::string mp_xml::get_route(const xmlNode *node)
     return route_value;
 }
 
+std::string mp_xml::get_route(const xmlNode *node)
+{
+    std::string auth;
+    return get_route(node, auth);
+}
 
 const xmlNode* mp_xml::jump_to_children(const xmlNode* node,
                                           int xml_node_type)
@@ -200,10 +213,16 @@ void mp_xml::check_empty(const xmlNode *node)
     if (node)
     {
         const xmlNode *n;
+        const struct _xmlAttr *attr;
+        std::string extra;
+        for (attr = node->properties; attr; attr = attr->next)
+            if (!strcmp((const char *) attr->name, "type"))
+                extra = " of type " + get_text(attr);
         for (n = node->children; n; n = n->next)
             if (n->type == XML_ELEMENT_NODE)
                 throw mp::XMLError("No child elements allowed inside element "
-                                    + std::string((const char *) node->name));
+                                   + std::string((const char *) node->name)
+                                   + extra);
     }
 }
 
